@@ -19,7 +19,7 @@
 
 /* Target triple LLVM will generate code for by default */
 /* Doesn't use `cmakedefine` because it is allowed to be empty. */
-#define LLVM_DEFAULT_TARGET_TRIPLE "sparc64-unknown-netbsd10.0"
+#define LLVM_DEFAULT_TARGET_TRIPLE "armv7-unknown-netbsdelf11.99-eabihf"
 
 /* Define if threads enabled */
 #define LLVM_ENABLE_THREADS 1
@@ -28,28 +28,28 @@
 #define LLVM_HAS_ATOMICS 1
 
 /* Host triple LLVM will be executed on */
-#define LLVM_HOST_TRIPLE "sparc64-unknown-netbsd10.0"
+#define LLVM_HOST_TRIPLE "armv7-unknown-netbsdelf11.99-eabihf"
 
 /* LLVM architecture name for the native architecture, if available */
-#define LLVM_NATIVE_ARCH Sparc
+#define LLVM_NATIVE_ARCH ARM
 
 /* LLVM name for the native AsmParser init function, if available */
-#define LLVM_NATIVE_ASMPARSER LLVMInitializeSparcAsmParser
+#define LLVM_NATIVE_ASMPARSER LLVMInitializeARMAsmParser
 
 /* LLVM name for the native AsmPrinter init function, if available */
-#define LLVM_NATIVE_ASMPRINTER LLVMInitializeSparcAsmPrinter
+#define LLVM_NATIVE_ASMPRINTER LLVMInitializeARMAsmPrinter
 
 /* LLVM name for the native Disassembler init function, if available */
-#define LLVM_NATIVE_DISASSEMBLER LLVMInitializeSparcDisassembler
+#define LLVM_NATIVE_DISASSEMBLER LLVMInitializeARMDisassembler
 
 /* LLVM name for the native Target init function, if available */
-#define LLVM_NATIVE_TARGET LLVMInitializeSparcTarget
+#define LLVM_NATIVE_TARGET LLVMInitializeARMTarget
 
 /* LLVM name for the native TargetInfo init function, if available */
-#define LLVM_NATIVE_TARGETINFO LLVMInitializeSparcTargetInfo
+#define LLVM_NATIVE_TARGETINFO LLVMInitializeARMTargetInfo
 
 /* LLVM name for the native target MC init function, if available */
-#define LLVM_NATIVE_TARGETMC LLVMInitializeSparcTargetMC
+#define LLVM_NATIVE_TARGETMC LLVMInitializeARMTargetMC
 
 /* LLVM name for the native target MCA init function, if available */
 /* #undef LLVM_NATIVE_TARGETMCA */
@@ -64,7 +64,7 @@
 #define LLVM_HAS_ARC_TARGET 0
 
 /* Define if the ARM target is built in */
-#define LLVM_HAS_ARM_TARGET 0
+#define LLVM_HAS_ARM_TARGET 1
 
 /* Define if the AVR target is built in */
 #define LLVM_HAS_AVR_TARGET 0
@@ -106,7 +106,7 @@
 #define LLVM_HAS_RISCV_TARGET 0
 
 /* Define if the Sparc target is built in */
-#define LLVM_HAS_SPARC_TARGET 1
+#define LLVM_HAS_SPARC_TARGET 0
 
 /* Define if the SPIRV target is built in */
 #define LLVM_HAS_SPIRV_TARGET 0
@@ -142,16 +142,16 @@
 #define LLVM_USE_PERF 0
 
 /* Major version of the LLVM API */
-#define LLVM_VERSION_MAJOR 18
+#define LLVM_VERSION_MAJOR 19
 
 /* Minor version of the LLVM API */
 #define LLVM_VERSION_MINOR 1
 
 /* Patch version of the LLVM API */
-#define LLVM_VERSION_PATCH 8
+#define LLVM_VERSION_PATCH 7
 
 /* LLVM version string */
-#define LLVM_VERSION_STRING "18.1.8"
+#define LLVM_VERSION_STRING "19.1.7"
 
 /* Whether LLVM records statistics for use with GetStatistics(),
  * PrintStatistics() or PrintStatisticsJSON()
@@ -198,4 +198,7 @@
 /* Define if plugins enabled */
 #define LLVM_ENABLE_PLUGINS
 
+/* Define if logf128 is available */
+/* #undef LLVM_HAS_LOGF128 */
+
 #endif
