@@ -13,6 +13,7 @@
 #ifndef LLVM_SUPPORT_ENDIAN_H
 #define LLVM_SUPPORT_ENDIAN_H
 
+#include "llvm/ADT/bit.h"
 #include "llvm/Support/Compiler.h"
 #include "llvm/Support/SwapByteOrder.h"
 #include <cassert>
@@ -24,8 +25,6 @@
 namespace llvm {
 namespace support {
 
-enum endianness {big, little, native};
-
 // These are named values for common alignments.
 enum {aligned = 0, unaligned = 1};
 
@@ -41,26 +40,22 @@ struct PickAlignment {
 
 namespace endian {
 
-constexpr endianness system_endianness() {
-  return sys::IsBigEndianHost ? big : little;
-}
-
 template <typename value_type>
-inline value_type byte_swap(value_type value, endianness endian) {
-  if ((endian != native) && (endian != system_endianness()))
+[[nodiscard]] inline value_type byte_swap(value_type value, endianness endian) {
+  if (endian != llvm::endianness::native)
     sys::swapByteOrder(value);
   return value;
 }
 
 /// Swap the bytes of value to match the given endianness.
-template<typename value_type, endianness endian>
-inline value_type byte_swap(value_type value) {
+template <typename value_type, endianness endian>
+[[nodiscard]] inline value_type byte_swap(value_type value) {
   return byte_swap(value, endian);
 }
 
 /// Read a value of a particular endianness from memory.
-template <typename value_type, std::size_t alignment>
-inline value_type read(const void *memory, endianness endian) {
+template <typename value_type, std::size_t alignment = unaligned>
+[[nodiscard]] inline value_type read(const void *memory, endianness endian) {
   value_type ret;
 
   memcpy(&ret,
@@ -70,30 +65,30 @@ inline value_type read(const void *memory, endianness endian) {
   return byte_swap<value_type>(ret, endian);
 }
 
-template<typename value_type,
-         endianness endian,
-         std::size_t alignment>
-inline value_type read(const void *memory) {
+template <typename value_type, endianness endian, std::size_t alignment>
+[[nodiscard]] inline value_type read(const void *memory) {
   return read<value_type, alignment>(memory, endian);
 }
 
 /// Read a value of a particular endianness from a buffer, and increment the
 /// buffer past that value.
-template <typename value_type, std::size_t alignment, typename CharT>
-inline value_type readNext(const CharT *&memory, endianness endian) {
+template <typename value_type, std::size_t alignment = unaligned,
+          typename CharT>
+[[nodiscard]] inline value_type readNext(const CharT *&memory,
+                                         endianness endian) {
   value_type ret = read<value_type, alignment>(memory, endian);
   memory += sizeof(value_type);
   return ret;
 }
 
-template<typename value_type, endianness endian, std::size_t alignment,
-         typename CharT>
-inline value_type readNext(const CharT *&memory) {
+template <typename value_type, endianness endian,
+          std::size_t alignment = unaligned, typename CharT>
+[[nodiscard]] inline value_type readNext(const CharT *&memory) {
   return readNext<value_type, alignment, CharT>(memory, endian);
 }
 
 /// Write a value to memory with a particular endianness.
-template <typename value_type, std::size_t alignment>
+template <typename value_type, std::size_t alignment = unaligned>
 inline void write(void *memory, value_type value, endianness endian) {
   value = byte_swap<value_type>(value, endian);
   memcpy(LLVM_ASSUME_ALIGNED(
@@ -108,13 +103,29 @@ inline void write(void *memory, value_type value) {
   write<value_type, alignment>(memory, value, endian);
 }
 
+/// Write a value of a particular endianness, and increment the buffer past that
+/// value.
+template <typename value_type, std::size_t alignment = unaligned,
+          typename CharT>
+inline void writeNext(CharT *&memory, value_type value, endianness endian) {
+  write(memory, value, endian);
+  memory += sizeof(value_type);
+}
+
+template <typename value_type, endianness endian,
+          std::size_t alignment = unaligned, typename CharT>
+inline void writeNext(CharT *&memory, value_type value) {
+  writeNext<value_type, alignment, CharT>(memory, value, endian);
+}
+
 template <typename value_type>
 using make_unsigned_t = std::make_unsigned_t<value_type>;
 
 /// Read a value of a particular endianness from memory, for a location
 /// that starts at the given bit offset within the first byte.
 template <typename value_type, endianness endian, std::size_t alignment>
-inline value_type readAtBitAlignment(const void *memory, uint64_t startBit) {
+[[nodiscard]] inline value_type readAtBitAlignment(const void *memory,
+                                                   uint64_t startBit) {
   assert(startBit < 8);
   if (startBit == 0)
     return read<value_type, endian, alignment>(memory);
@@ -267,125 +278,164 @@ public:
 } // end namespace detail
 
 using ulittle16_t =
-    detail::packed_endian_specific_integral<uint16_t, little, unaligned>;
+    detail::packed_endian_specific_integral<uint16_t, llvm::endianness::little,
+                                            unaligned>;
 using ulittle32_t =
-    detail::packed_endian_specific_integral<uint32_t, little, unaligned>;
+    detail::packed_endian_specific_integral<uint32_t, llvm::endianness::little,
+                                            unaligned>;
 using ulittle64_t =
-    detail::packed_endian_specific_integral<uint64_t, little, unaligned>;
+    detail::packed_endian_specific_integral<uint64_t, llvm::endianness::little,
+                                            unaligned>;
 
 using little16_t =
-    detail::packed_endian_specific_integral<int16_t, little, unaligned>;
+    detail::packed_endian_specific_integral<int16_t, llvm::endianness::little,
+                                            unaligned>;
 using little32_t =
-    detail::packed_endian_specific_integral<int32_t, little, unaligned>;
+    detail::packed_endian_specific_integral<int32_t, llvm::endianness::little,
+                                            unaligned>;
 using little64_t =
-    detail::packed_endian_specific_integral<int64_t, little, unaligned>;
+    detail::packed_endian_specific_integral<int64_t, llvm::endianness::little,
+                                            unaligned>;
 
 using aligned_ulittle16_t =
-    detail::packed_endian_specific_integral<uint16_t, little, aligned>;
+    detail::packed_endian_specific_integral<uint16_t, llvm::endianness::little,
+                                            aligned>;
 using aligned_ulittle32_t =
-    detail::packed_endian_specific_integral<uint32_t, little, aligned>;
+    detail::packed_endian_specific_integral<uint32_t, llvm::endianness::little,
+                                            aligned>;
 using aligned_ulittle64_t =
-    detail::packed_endian_specific_integral<uint64_t, little, aligned>;
+    detail::packed_endian_specific_integral<uint64_t, llvm::endianness::little,
+                                            aligned>;
 
 using aligned_little16_t =
-    detail::packed_endian_specific_integral<int16_t, little, aligned>;
+    detail::packed_endian_specific_integral<int16_t, llvm::endianness::little,
+                                            aligned>;
 using aligned_little32_t =
-    detail::packed_endian_specific_integral<int32_t, little, aligned>;
+    detail::packed_endian_specific_integral<int32_t, llvm::endianness::little,
+                                            aligned>;
 using aligned_little64_t =
-    detail::packed_endian_specific_integral<int64_t, little, aligned>;
+    detail::packed_endian_specific_integral<int64_t, llvm::endianness::little,
+                                            aligned>;
 
 using ubig16_t =
-    detail::packed_endian_specific_integral<uint16_t, big, unaligned>;
+    detail::packed_endian_specific_integral<uint16_t, llvm::endianness::big,
+                                            unaligned>;
 using ubig32_t =
-    detail::packed_endian_specific_integral<uint32_t, big, unaligned>;
+    detail::packed_endian_specific_integral<uint32_t, llvm::endianness::big,
+                                            unaligned>;
 using ubig64_t =
-    detail::packed_endian_specific_integral<uint64_t, big, unaligned>;
+    detail::packed_endian_specific_integral<uint64_t, llvm::endianness::big,
+                                            unaligned>;
 
 using big16_t =
-    detail::packed_endian_specific_integral<int16_t, big, unaligned>;
+    detail::packed_endian_specific_integral<int16_t, llvm::endianness::big,
+                                            unaligned>;
 using big32_t =
-    detail::packed_endian_specific_integral<int32_t, big, unaligned>;
+    detail::packed_endian_specific_integral<int32_t, llvm::endianness::big,
+                                            unaligned>;
 using big64_t =
-    detail::packed_endian_specific_integral<int64_t, big, unaligned>;
+    detail::packed_endian_specific_integral<int64_t, llvm::endianness::big,
+                                            unaligned>;
 
 using aligned_ubig16_t =
-    detail::packed_endian_specific_integral<uint16_t, big, aligned>;
+    detail::packed_endian_specific_integral<uint16_t, llvm::endianness::big,
+                                            aligned>;
 using aligned_ubig32_t =
-    detail::packed_endian_specific_integral<uint32_t, big, aligned>;
+    detail::packed_endian_specific_integral<uint32_t, llvm::endianness::big,
+                                            aligned>;
 using aligned_ubig64_t =
-    detail::packed_endian_specific_integral<uint64_t, big, aligned>;
+    detail::packed_endian_specific_integral<uint64_t, llvm::endianness::big,
+                                            aligned>;
 
 using aligned_big16_t =
-    detail::packed_endian_specific_integral<int16_t, big, aligned>;
+    detail::packed_endian_specific_integral<int16_t, llvm::endianness::big,
+                                            aligned>;
 using aligned_big32_t =
-    detail::packed_endian_specific_integral<int32_t, big, aligned>;
+    detail::packed_endian_specific_integral<int32_t, llvm::endianness::big,
+                                            aligned>;
 using aligned_big64_t =
-    detail::packed_endian_specific_integral<int64_t, big, aligned>;
+    detail::packed_endian_specific_integral<int64_t, llvm::endianness::big,
+                                            aligned>;
 
 using unaligned_uint16_t =
-    detail::packed_endian_specific_integral<uint16_t, native, unaligned>;
+    detail::packed_endian_specific_integral<uint16_t, llvm::endianness::native,
+                                            unaligned>;
 using unaligned_uint32_t =
-    detail::packed_endian_specific_integral<uint32_t, native, unaligned>;
+    detail::packed_endian_specific_integral<uint32_t, llvm::endianness::native,
+                                            unaligned>;
 using unaligned_uint64_t =
-    detail::packed_endian_specific_integral<uint64_t, native, unaligned>;
+    detail::packed_endian_specific_integral<uint64_t, llvm::endianness::native,
+                                            unaligned>;
 
 using unaligned_int16_t =
-    detail::packed_endian_specific_integral<int16_t, native, unaligned>;
+    detail::packed_endian_specific_integral<int16_t, llvm::endianness::native,
+                                            unaligned>;
 using unaligned_int32_t =
-    detail::packed_endian_specific_integral<int32_t, native, unaligned>;
+    detail::packed_endian_specific_integral<int32_t, llvm::endianness::native,
+                                            unaligned>;
 using unaligned_int64_t =
-    detail::packed_endian_specific_integral<int64_t, native, unaligned>;
+    detail::packed_endian_specific_integral<int64_t, llvm::endianness::native,
+                                            unaligned>;
 
 template <typename T>
-using little_t = detail::packed_endian_specific_integral<T, little, unaligned>;
+using little_t =
+    detail::packed_endian_specific_integral<T, llvm::endianness::little,
+                                            unaligned>;
 template <typename T>
-using big_t = detail::packed_endian_specific_integral<T, big, unaligned>;
+using big_t = detail::packed_endian_specific_integral<T, llvm::endianness::big,
+                                                      unaligned>;
 
 template <typename T>
 using aligned_little_t =
-    detail::packed_endian_specific_integral<T, little, aligned>;
+    detail::packed_endian_specific_integral<T, llvm::endianness::little,
+                                            aligned>;
 template <typename T>
-using aligned_big_t = detail::packed_endian_specific_integral<T, big, aligned>;
+using aligned_big_t =
+    detail::packed_endian_specific_integral<T, llvm::endianness::big, aligned>;
 
 namespace endian {
 
-template <typename T> inline T read(const void *P, endianness E) {
-  return read<T, unaligned>(P, E);
-}
-
-template <typename T, endianness E> inline T read(const void *P) {
+template <typename T, endianness E> [[nodiscard]] inline T read(const void *P) {
   return *(const detail::packed_endian_specific_integral<T, E, unaligned> *)P;
 }
 
-inline uint16_t read16(const void *P, endianness E) {
+[[nodiscard]] inline uint16_t read16(const void *P, endianness E) {
   return read<uint16_t>(P, E);
 }
-inline uint32_t read32(const void *P, endianness E) {
+[[nodiscard]] inline uint32_t read32(const void *P, endianness E) {
   return read<uint32_t>(P, E);
 }
-inline uint64_t read64(const void *P, endianness E) {
+[[nodiscard]] inline uint64_t read64(const void *P, endianness E) {
   return read<uint64_t>(P, E);
 }
 
-template <endianness E> inline uint16_t read16(const void *P) {
+template <endianness E> [[nodiscard]] inline uint16_t read16(const void *P) {
   return read<uint16_t, E>(P);
 }
-template <endianness E> inline uint32_t read32(const void *P) {
+template <endianness E> [[nodiscard]] inline uint32_t read32(const void *P) {
   return read<uint32_t, E>(P);
 }
-template <endianness E> inline uint64_t read64(const void *P) {
+template <endianness E> [[nodiscard]] inline uint64_t read64(const void *P) {
   return read<uint64_t, E>(P);
 }
 
-inline uint16_t read16le(const void *P) { return read16<little>(P); }
-inline uint32_t read32le(const void *P) { return read32<little>(P); }
-inline uint64_t read64le(const void *P) { return read64<little>(P); }
-inline uint16_t read16be(const void *P) { return read16<big>(P); }
-inline uint32_t read32be(const void *P) { return read32<big>(P); }
-inline uint64_t read64be(const void *P) { return read64<big>(P); }
-
-template <typename T> inline void write(void *P, T V, endianness E) {
-  write<T, unaligned>(P, V, E);
+[[nodiscard]] inline uint16_t read16le(const void *P) {
+  return read16<llvm::endianness::little>(P);
+}
+[[nodiscard]] inline uint32_t read32le(const void *P) {
+  return read32<llvm::endianness::little>(P);
+}
+[[nodiscard]] inline uint64_t read64le(const void *P) {
+  return read64<llvm::endianness::little>(P);
+}
+[[nodiscard]] inline uint16_t read16be(const void *P) {
+  return read16<llvm::endianness::big>(P);
+}
+[[nodiscard]] inline uint32_t read32be(const void *P) {
+  return read32<llvm::endianness::big>(P);
+}
+[[nodiscard]] inline uint64_t read64be(const void *P) {
+  return read64<llvm::endianness::big>(P);
 }
 
 template <typename T, endianness E> inline void write(void *P, T V) {
@@ -412,12 +462,24 @@ template <endianness E> inline void write64(void *P, uint64_t V) {
   write<uint64_t, E>(P, V);
 }
 
-inline void write16le(void *P, uint16_t V) { write16<little>(P, V); }
-inline void write32le(void *P, uint32_t V) { write32<little>(P, V); }
-inline void write64le(void *P, uint64_t V) { write64<little>(P, V); }
-inline void write16be(void *P, uint16_t V) { write16<big>(P, V); }
-inline void write32be(void *P, uint32_t V) { write32<big>(P, V); }
-inline void write64be(void *P, uint64_t V) { write64<big>(P, V); }
+inline void write16le(void *P, uint16_t V) {
+  write16<llvm::endianness::little>(P, V);
+}
+inline void write32le(void *P, uint32_t V) {
+  write32<llvm::endianness::little>(P, V);
+}
+inline void write64le(void *P, uint64_t V) {
+  write64<llvm::endianness::little>(P, V);
+}
+inline void write16be(void *P, uint16_t V) {
+  write16<llvm::endianness::big>(P, V);
+}
+inline void write32be(void *P, uint32_t V) {
+  write32<llvm::endianness::big>(P, V);
+}
+inline void write64be(void *P, uint64_t V) {
+  write64<llvm::endianness::big>(P, V);
+}
 
 } // end namespace endian
 
